import processing.core.*;
import processing.video.*;
import processing.blobDetection.*;

public class SimpleBlobTracking extends PApplet {

	Capture cam;
	BlobDetection theBlobDetection;
	PImage img;
	PImage diff;
	PImage prev;
	
	boolean newFrame = false;

	public SimpleBlobTracking() {
		super();
	}

	public void settings() {
		size( 640, 480 );
	}

	public void setup() {
		
		cam = new Capture( this, width, height, 15 );
		cam.start();

		img = new PImage( 320, 240 );
		diff = new PImage( img.width, img.height );
		prev = new PImage( img.width, img.height );
		
		theBlobDetection = new BlobDetection( img.width, img.height );
		theBlobDetection.setPosDiscrimination( true );
		theBlobDetection.setThreshold( 0.1f );
		
	}
	
	public void captureEvent(Capture cam) {
		cam.read();
		newFrame = true;
	}

	public void draw() {
		if (newFrame) {
			newFrame = false;
			image(cam, 0, 0, width, height);
			
			// loading new image
			img.copy( cam, 0, 0, cam.width, cam.height, 0, 0, img.width, img.height );
			
			processDiff( 30 );
			
			fastblur( diff, 3 );
			theBlobDetection.computeBlobs( diff.pixels );
			
			drawBlobsAndEdges( true, true );
			
			image( diff, 0,0 );
			
		}
	}
	
	private void processDiff( int tolerance ) {
		
		int pxnum = img.width * img.height;
		img.loadPixels();
		prev.loadPixels();
		diff.loadPixels();
		int white = color( 255,255,255 );
		int black = color( 0,0,0 );
		for ( int i = 0; i < pxnum; ++i ) {
			int ic = img.pixels[ i ];
			int pc = prev.pixels[ i ];
			if (
				abs( ( ( ic >> 16) & 0xFF ) - ( ( pc >> 16) & 0xFF ) ) > tolerance ||
				abs( ( ( ic >> 8) & 0xFF ) - ( ( pc >> 8) & 0xFF ) ) > tolerance ||
				abs( ( ic & 0xFF ) - ( pc & 0xFF ) ) > tolerance
			) {
				diff.pixels[ i ] = white;
			} else {
				diff.pixels[ i ] = black;
			}
			// backuping at the same time
			prev.pixels[ i ] = ic;
		}
		prev.updatePixels();
		diff.updatePixels();
		
	}

	private void drawBlobsAndEdges(boolean drawBlobs, boolean drawEdges) {
		noFill();
		Blob b;
		EdgeVertex eA, eB;
		for (int n = 0; n < theBlobDetection.getBlobNb(); n++) {
			b = theBlobDetection.getBlob(n);
			if (b != null) {
				// Edges
				if (drawEdges) {
					strokeWeight(3);
					stroke(0, 255, 0);
					for (int m = 0; m < b.getEdgeNb(); m++) {
						eA = b.getEdgeVertexA(m);
						eB = b.getEdgeVertexB(m);
						if (eA != null && eB != null)
							line(eA.x * width, eA.y * height, eB.x * width,
									eB.y * height);
					}
				}
				// Blobs
				if (drawBlobs) {
					strokeWeight(1);
					stroke(255, 0, 0);
					rect(b.xMin * width, b.yMin * height, b.w * width, b.h
							* height);
				}
			}
		}
	}

	// ==================================================
	// Super Fast Blur v1.1
	// by Mario Klingemann
	// <http://incubator.quasimondo.com>
	// ==================================================
	private void fastblur(PImage img, int radius) {
		
		if (radius < 1) {
			return;
		}
		int w = img.width;
		int h = img.height;
		int wm = w - 1;
		int hm = h - 1;
		int wh = w * h;
		int div = radius + radius + 1;
		int r[] = new int[wh];
		int g[] = new int[wh];
		int b[] = new int[wh];
		int rsum, gsum, bsum, x, y, i, p, p1, p2, yp, yi, yw;
		int vmin[] = new int[max(w, h)];
		int vmax[] = new int[max(w, h)];
		int[] pix = img.pixels;
		int dv[] = new int[256 * div];
		for (i = 0; i < 256 * div; i++) {
			dv[i] = (i / div);
		}
		yw = yi = 0;

		for (y = 0; y < h; y++) {
			rsum = gsum = bsum = 0;
			for (i = -radius; i <= radius; i++) {
				p = pix[yi + min(wm, max(i, 0))];
				rsum += (p & 0xff0000) >> 16;
				gsum += (p & 0x00ff00) >> 8;
				bsum += p & 0x0000ff;
			}
			for (x = 0; x < w; x++) {
				r[yi] = dv[rsum];
				g[yi] = dv[gsum];
				b[yi] = dv[bsum];
				if (y == 0) {
					vmin[x] = min(x + radius + 1, wm);
					vmax[x] = max(x - radius, 0);
				}
				p1 = pix[yw + vmin[x]];
				p2 = pix[yw + vmax[x]];
				rsum += ((p1 & 0xff0000) - (p2 & 0xff0000)) >> 16;
				gsum += ((p1 & 0x00ff00) - (p2 & 0x00ff00)) >> 8;
				bsum += (p1 & 0x0000ff) - (p2 & 0x0000ff);
				yi++;
			}
			yw += w;
		}

		for (x = 0; x < w; x++) {
			rsum = gsum = bsum = 0;
			yp = -radius * w;
			for (i = -radius; i <= radius; i++) {
				yi = max(0, yp) + x;
				rsum += r[yi];
				gsum += g[yi];
				bsum += b[yi];
				yp += w;
			}
			yi = x;
			for (y = 0; y < h; y++) {
				pix[yi] = 0xff000000 | (dv[rsum] << 16) | (dv[gsum] << 8) | dv[bsum];
				if (x == 0) {
					vmin[y] = min(y + radius + 1, hm) * w;
					vmax[y] = max(y - radius, 0) * w;
				}
				p1 = x + vmin[y];
				p2 = x + vmax[y];
				rsum += r[p1] - r[p2];
				gsum += g[p1] - g[p2];
				bsum += b[p1] - b[p2];
				yi += w;
			}
		}
	}

	public static void main(String args[]) {
		PApplet.main(new String[] { "SimpleBlobTracking" });
	}

}
